@@ -16,28 +16,30 @@ Choose parameter set with best AICc score.
 
 """
 
+import sys
+from typing import List, Dict
 import logging
 import operator
 from collections import OrderedDict
 
 import numpy as np
 import sympy
+from sympy import Symbol
 from tinydb import where
 from pycalphad import Database, Model, variables as v
-from pycalphad.io.database import Species
 
 import espei.refdata
-from espei.core_utils import get_data, get_samples, get_weights
+from espei.database_utils import initialize_database
+from espei.core_utils import get_data
+from espei.error_functions.non_equilibrium_thermochemical_error import get_prop_samples
 from espei.parameter_selection.model_building import build_candidate_models
 from espei.parameter_selection.selection import select_model
-from espei.parameter_selection.ternary_parameters import build_ternary_feature_matrix
-from espei.parameter_selection.utils import get_data_quantities, feature_transforms
+from espei.parameter_selection.utils import get_data_quantities, feature_transforms, _get_sample_condition_dicts
 from espei.sublattice_tools import generate_symmetric_group, generate_interactions, \
     tuplify, interaction_test, endmembers_from_interaction, generate_endmembers
-from espei.utils import PickleableTinyDB, sigfigs
+from espei.utils import PickleableTinyDB, sigfigs, extract_aliases
 
-
-TRACE = 15  # TRACE logging level
+_log = logging.getLogger(__name__)
 
 
 def _param_present_in_database(dbf, phase_name, configuration, param_type):
@@ -51,29 +53,30 @@ def _param_present_in_database(dbf, phase_name, configuration, param_type):
         return True
 
 
-def _build_feature_matrix(prop, features, desired_data):
+def _build_feature_matrix(sample_condition_dicts: List[Dict[Symbol, float]], symbolic_coefficients: List[Symbol]):
     """
-    Return an MxN matrix of M data sample and N features.
+    Builds A for solving x = A\\b. A is an MxN matrix of M sampled data points and N is the symbolic coefficients.
 
     Parameters
     ----------
-    prop : str
-        String name of the property, e.g. 'HM_MIX'
-    features : tuple
-        Tuple of SymPy parameters that can be fit for this property.
-    desired_data : dict
-        Full dataset dictionary containing values, conditions, etc.
+    sample_condition_dicts : List[Dict[Symbol, float]]
+        List of length ``M`` containing the conditions (T, P, YS, Z, V_I, V_J, V_K) for
+        each sampled point.
+    symbolic_coefficients : List[Symbol]
+        Symbolic coefficients of length ```N`` (e.g. ``v.T``, ``YS``) of the features
+        corresponding to the variables that will be fit.
 
     Returns
     -------
-    numpy.ndarray
-        An MxN matrix of M samples (from desired data) and N features.
-
+    ArrayLike
+        MxN array of coefficients with sampled data conditions plugged in.
     """
-    transformed_features = [feature_transforms[prop](i) for i in features]
-    all_samples = get_samples(desired_data)
-    feature_matrix = np.empty((len(all_samples), len(transformed_features)), dtype=np.float)
-    feature_matrix[:, :] = [[trans_feat.subs({v.T: temp, 'YS': compf[0], 'Z': compf[1]}).evalf() for trans_feat in transformed_features] for temp, compf in all_samples]
+    M = len(sample_condition_dicts)
+    N = len(symbolic_coefficients)
+    feature_matrix = np.empty((M, N))
+    for i in range(M):
+        for j in range(N):
+            feature_matrix[i, j] = symbolic_coefficients[j].subs(sample_condition_dicts[i])
     return feature_matrix
 
 
@@ -117,7 +120,7 @@ def fit_formation_energy(dbf, comps, phase_name, configuration, symmetry, datase
     """
     aicc_feature_factors = aicc_phase_penalty if aicc_phase_penalty is not None else {}
     if interaction_test(configuration):
-        logging.debug('ENDMEMBERS FROM INTERACTION: {}'.format(endmembers_from_interaction(configuration)))
+        _log.debug('ENDMEMBERS FROM INTERACTION: %s', endmembers_from_interaction(configuration))
         fitting_steps = (["CPM_FORM", "CPM_MIX"], ["SM_FORM", "SM_MIX"], ["HM_FORM", "HM_MIX"])
 
     else:
@@ -152,24 +155,29 @@ def fit_formation_energy(dbf, comps, phase_name, configuration, symmetry, datase
         feature_type = desired_props[0].split('_')[0]  # HM_FORM -> HM
         aicc_factor = aicc_feature_factors.get(feature_type, 1.0)
         desired_data = get_data(comps, phase_name, configuration, symmetry, datasets, desired_props)
-        logging.log(TRACE, '{}: datasets found: {}'.format(desired_props, len(desired_data)))
+        _log.trace('%s: datasets found: %s', desired_props, len(desired_data))
         if len(desired_data) > 0:
-            # Ravelled weights for all data
-            weights = get_weights(desired_data)
+            config_tup = tuple(map(tuplify, configuration))
+            calculate_dict = get_prop_samples(desired_data, config_tup)
+            sample_condition_dicts = _get_sample_condition_dicts(calculate_dict, list(map(len, config_tup)))
+            weights = calculate_dict['weights']
+            assert len(sample_condition_dicts) == len(weights)
 
             # We assume all properties in the same fitting step have the same
             # features (all CPM, all HM, etc., but different ref states).
             # data quantities are the same for each candidate model and can be computed up front
-            data_qtys = get_data_quantities(feature_type, fixed_model, fixed_portions, desired_data)
+            data_qtys = get_data_quantities(feature_type, fixed_model, fixed_portions, desired_data, sample_condition_dicts)
 
             # build the candidate model transformation matrix and response vector (A, b in Ax=b)
             feature_matricies = []
             data_quantities = []
-            for candidate_model in candidate_models_features[desired_props[0]]:
+            for candidate_coefficients in candidate_models_features[desired_props[0]]:
+                # Map coeffiecients in G to coefficients in the feature_type (H, S, CP)
+                transformed_coefficients = list(map(feature_transforms[feature_type], candidate_coefficients))
                 if interaction_test(configuration, 3):
-                    feature_matricies.append(build_ternary_feature_matrix(desired_props[0], candidate_model, desired_data))
+                    feature_matricies.append(_build_feature_matrix(sample_condition_dicts, transformed_coefficients))
                 else:
-                    feature_matricies.append(_build_feature_matrix(desired_props[0], candidate_model, desired_data))
+                    feature_matricies.append(_build_feature_matrix(sample_condition_dicts, transformed_coefficients))
                 data_quantities.append(data_qtys)
 
             # provide candidate models and get back a selected model.
@@ -177,7 +185,7 @@ def fit_formation_energy(dbf, comps, phase_name, configuration, symmetry, datase
             selected_features, selected_values = selected_model
             parameters.update(zip(*(selected_features, selected_values)))
             # Add these parameters to be fixed for the next fitting step
-            fixed_portion = np.array(selected_features, dtype=np.object)
+            fixed_portion = np.array(selected_features, dtype=np.object_)
             fixed_portion = np.dot(fixed_portion, selected_values)
             fixed_portions.append(fixed_portion)
     return parameters
@@ -231,24 +239,23 @@ def fit_ternary_interactions(dbf, phase_name, symmetry, endmembers, datasets, ri
     """
     numdigits = 6  # number of significant figures, might cause rounding errors
     interactions = generate_interactions(endmembers, order=3, symmetry=symmetry)
-    logging.log(TRACE, '{0} distinct ternary interactions'.format(len(interactions)))
+    _log.trace('%s distinct ternary interactions', len(interactions))
     for interaction in interactions:
         ixx = interaction
         config = tuple(map(tuplify, ixx))
         if _param_present_in_database(dbf, phase_name, config, 'L'):
-            logging.log(TRACE, 'INTERACTION: {} already in Database'.format(ixx))
-            print('INTERACTION: {} already in Database'.format(ixx))
+            _log.warning('INTERACTION: %s already in Database. Skipping.', ixx)
             continue
         else:
-            logging.log(TRACE, 'INTERACTION: {}'.format(ixx))
+            _log.trace('INTERACTION: %s', ixx)
         parameters = fit_formation_energy(dbf, sorted(dbf.elements), phase_name, ixx, symmetry, datasets, ridge_alpha, aicc_phase_penalty=aicc_phase_penalty)
         # Organize parameters by polynomial degree
-        degree_polys = np.zeros(3, dtype=np.object)
-        YS = sympy.Symbol('YS')
+        degree_polys = np.zeros(3, dtype=np.object_)
+        YS = Symbol('YS')
         # asymmetric parameters should have Mugiannu V_I/V_J/V_K, while symmetric just has YS
-        is_asymmetric = any([(k.has(sympy.Symbol('V_I'))) and (v != 0) for k, v in parameters.items()])
+        is_asymmetric = any([(k.has(Symbol('V_I'))) and (v != 0) for k, v in parameters.items()])
         if is_asymmetric:
-            params = [(2, YS*sympy.Symbol('V_K')), (1, YS*sympy.Symbol('V_J')), (0, YS*sympy.Symbol('V_I'))]  # (excess parameter degree, symbol) tuples
+            params = [(2, YS*Symbol('V_K')), (1, YS*Symbol('V_J')), (0, YS*Symbol('V_I'))]  # (excess parameter degree, symbol) tuples
         else:
             params = [(0, YS)]  # (excess parameter degree, symbol) tuples
         for degree, check_symbol in params:
@@ -258,7 +265,7 @@ def fit_ternary_interactions(dbf, phase_name, symmetry, endmembers, datasets, ri
                     if value != 0:
                         symbol_name = get_next_symbol(dbf)
                         dbf.symbols[symbol_name] = sigfigs(parameters[key], numdigits)
-                        parameters[key] = sympy.Symbol(symbol_name)
+                        parameters[key] = Symbol(symbol_name)
                     coef = parameters[key] * (key / check_symbol)
                     try:
                         coef = float(coef)
@@ -268,7 +275,7 @@ def fit_ternary_interactions(dbf, phase_name, symmetry, endmembers, datasets, ri
                     keys_to_remove.append(key)
             for key in keys_to_remove:
                 parameters.pop(key)
-        logging.log(TRACE, 'Polynomial coefs: {}'.format(degree_polys))
+        _log.trace('Polynomial coefs: %s', degree_polys)
         # Insert into database
         symmetric_interactions = generate_symmetric_group(interaction, symmetry)
         for degree in np.arange(degree_polys.shape[0]):
@@ -277,7 +284,7 @@ def fit_ternary_interactions(dbf, phase_name, symmetry, endmembers, datasets, ri
                     dbf.add_parameter('L', phase_name, tuple(map(tuplify, syminter)), degree, degree_polys[degree])
 
 
-def phase_fit(dbf, phase_name, symmetry, subl_model, site_ratios, datasets, refdata, ridge_alpha, aicc_penalty=None, aliases=None):
+def phase_fit(dbf, phase_name, symmetry, datasets, refdata, ridge_alpha, aicc_penalty=None, aliases=None):
     """Generate an initial CALPHAD model for a given phase and sublattice model.
 
     Parameters
@@ -288,10 +295,6 @@ def phase_fit(dbf, phase_name, symmetry, subl_model, site_ratios, datasets, refd
         Name of the phase.
     symmetry : [[int]]
         Sublattice model symmetry.
-    subl_model : [[str]]
-        Sublattice model for the phase of interest.
-    site_ratios : [float]
-        Number of sites in each sublattice, normalized to one atom.
     datasets : PickleableTinyDB
         All datasets to consider for the calculation.
     refdata : dict
@@ -302,9 +305,8 @@ def phase_fit(dbf, phase_name, symmetry, subl_model, site_ratios, datasets, refd
         with ordinary least squares regression. For now, the parameter is applied to all features.
     aicc_penalty : dict
         Map of phase name to feature to a multiplication factor for the AICc's parameter penalty.
-    aliases : [str]
-        Alternative phase names. Useful for matching against
-        reference data or other datasets. (Default value = None)
+    aliases : Dict[str, str]
+        Mapping of possible aliases to the Database phase names.
 
     Returns
     -------
@@ -316,26 +318,34 @@ def phase_fit(dbf, phase_name, symmetry, subl_model, site_ratios, datasets, refd
     aicc_phase_penalty = aicc_penalty.get(phase_name, {})
     if not hasattr(dbf, 'varcounter'):
         dbf.varcounter = 0
+    phase_obj = dbf.phases[phase_name]
+    # TODO: assumed pure elements - add proper support for Species objects
+    subl_model = [sorted([sp.name for sp in subl]) for subl in phase_obj.constituents]
+    site_ratios = phase_obj.sublattices
     # First fit endmembers
     all_em_count = len(generate_endmembers(subl_model))  # number of total endmembers
     endmembers = generate_endmembers(subl_model, symmetry)
     # Number of significant figures in parameters, might cause rounding errors
     numdigits = 6
     em_dict = {}
-    aliases = [] if aliases is None else aliases
-    aliases = sorted(set(aliases + [phase_name]))
-    logging.info('FITTING: {}'.format(phase_name))
-    logging.log(TRACE, '{0} endmembers ({1} distinct by symmetry)'.format(all_em_count, len(endmembers)))
+    # TODO: use the global aliases dictionary passed in as-is instead of converting it to a phase-local dict
+    # TODO: use the aliases dictionary in dataset queries to find relevant data
+    if aliases is None:
+        aliases = [phase_name]
+    else:
+        aliases = sorted([alias for alias, database_phase in aliases.items() if database_phase == phase_name])
+    _log.info('FITTING: %s', phase_name)
+    _log.trace('%s endmembers (%s distinct by symmetry)', all_em_count, len(endmembers))
 
     all_endmembers = []
     for endmember in endmembers:
         symmetric_endmembers = generate_symmetric_group(endmember, symmetry)
         all_endmembers.extend(symmetric_endmembers)
         if _param_present_in_database(dbf, phase_name, endmember, 'G'):
-            logging.log(TRACE, 'ENDMEMBER: {} already in Database'.format(endmember))
+            _log.trace('ENDMEMBER: %s already in Database. Skipping.', endmember)
             continue
         else:
-            logging.log(TRACE, 'ENDMEMBER: {}'.format(endmember))
+            _log.trace('ENDMEMBER: %s', endmember)
         # Some endmembers are fixed by our choice of standard lattice stabilities, e.g., SGTE91
         # If a (phase, pure component endmember) tuple is fixed, we should use that value instead of fitting
         endmember_comps = list(set(endmember))
@@ -360,7 +370,7 @@ def phase_fit(dbf, phase_name, symmetry, subl_model, site_ratios, datasets, refd
                     break
             if dbf.symbols.get(sym_name, None) is not None:
                 num_moles = sum([sites for elem, sites in zip(endmember, site_ratios) if elem != 'VA'])
-                fit_eq = num_moles * sympy.Symbol(sym_name)
+                fit_eq = num_moles * Symbol(sym_name)
         if fit_eq is None:
             # No reference lattice stability data -- we have to fit it
             parameters = fit_formation_energy(dbf, sorted(dbf.elements), phase_name, endmember, symmetry, datasets, ridge_alpha, aicc_phase_penalty=aicc_phase_penalty)
@@ -369,23 +379,23 @@ def phase_fit(dbf, phase_name, symmetry, subl_model, site_ratios, datasets, refd
                     continue
                 symbol_name = get_next_symbol(dbf)
                 dbf.symbols[symbol_name] = sigfigs(value, numdigits)
-                parameters[key] = sympy.Symbol(symbol_name)
+                parameters[key] = Symbol(symbol_name)
             fit_eq = sympy.Add(*[value * key for key, value in parameters.items()])
             ref = 0
             for subl, ratio in zip(endmember, site_ratios):
                 if subl == 'VA':
                     continue
                 subl = (subl.upper()*2)[:2]
-                ref = ref + ratio * sympy.Symbol('GHSER'+subl)
+                ref = ref + ratio * Symbol('GHSER'+subl)
             fit_eq += ref
-        logging.log(TRACE, 'SYMMETRIC_ENDMEMBERS: {}'.format(symmetric_endmembers))
+        _log.trace('SYMMETRIC_ENDMEMBERS: %s', symmetric_endmembers)
         for em in symmetric_endmembers:
             em_dict[em] = fit_eq
             dbf.add_parameter('G', phase_name, tuple(map(tuplify, em)), 0, fit_eq)
 
-    logging.log(TRACE, 'FITTING BINARY INTERACTIONS')
+    _log.trace('FITTING BINARY INTERACTIONS')
     bin_interactions = generate_interactions(all_endmembers, order=2, symmetry=symmetry)
-    logging.log(TRACE, '{0} distinct binary interactions'.format(len(bin_interactions)))
+    _log.trace('%s distinct binary interactions', len(bin_interactions))
     for interaction in bin_interactions:
         ixx = []
         for i in interaction:
@@ -396,22 +406,22 @@ def phase_fit(dbf, phase_name, symmetry, subl_model, site_ratios, datasets, refd
         ixx = tuple(ixx)
         config = tuple(map(tuplify, ixx))
         if _param_present_in_database(dbf, phase_name, config, 'L'):
-            logging.log(TRACE, 'INTERACTION: {} already in Database'.format(ixx))
+            _log.trace('INTERACTION: %s already in Database', ixx)
             continue
         else:
-            logging.log(TRACE, 'INTERACTION: {}'.format(ixx))
+            _log.trace('INTERACTION: %s', ixx)
         parameters = fit_formation_energy(dbf, sorted(dbf.elements), phase_name, ixx, symmetry, datasets, ridge_alpha, aicc_phase_penalty=aicc_phase_penalty)
         # Organize parameters by polynomial degree
-        degree_polys = np.zeros(10, dtype=np.object)
+        degree_polys = np.zeros(10, dtype=np.object_)
         for degree in reversed(range(10)):
-            check_symbol = sympy.Symbol('YS') * sympy.Symbol('Z')**degree
+            check_symbol = Symbol('YS') * Symbol('Z')**degree
             keys_to_remove = []
             for key, value in sorted(parameters.items(), key=str):
                 if key.has(check_symbol):
                     if value != 0:
                         symbol_name = get_next_symbol(dbf)
                         dbf.symbols[symbol_name] = sigfigs(parameters[key], numdigits)
-                        parameters[key] = sympy.Symbol(symbol_name)
+                        parameters[key] = Symbol(symbol_name)
                     coef = parameters[key] * (key / check_symbol)
                     try:
                         coef = float(coef)
@@ -421,7 +431,7 @@ def phase_fit(dbf, phase_name, symmetry, subl_model, site_ratios, datasets, refd
                     keys_to_remove.append(key)
             for key in keys_to_remove:
                 parameters.pop(key)
-        logging.log(TRACE, 'Polynomial coefs: {}'.format(degree_polys))
+        _log.trace('Polynomial coefs: %s', degree_polys)
         # Insert into database
         symmetric_interactions = generate_symmetric_group(interaction, symmetry)
         for degree in np.arange(degree_polys.shape[0]):
@@ -429,7 +439,7 @@ def phase_fit(dbf, phase_name, symmetry, subl_model, site_ratios, datasets, refd
                 for syminter in symmetric_interactions:
                     dbf.add_parameter('L', phase_name, tuple(map(tuplify, syminter)), degree, degree_polys[degree])
 
-    logging.log(TRACE, 'FITTING TERNARY INTERACTIONS')
+    _log.trace('FITTING TERNARY INTERACTIONS')
     fit_ternary_interactions(dbf, phase_name, symmetry, all_endmembers, datasets, aicc_phase_penalty=aicc_phase_penalty)
     if hasattr(dbf, 'varcounter'):
         del dbf.varcounter
@@ -462,55 +472,17 @@ def generate_parameters(phase_models, datasets, ref_state, excess_model, ridge_a
     pycalphad.Database
 
     """
-    logging.info('Generating parameters.')
-    logging.log(TRACE, f'Found the following user reference states: {espei.refdata.INSERTED_USER_REFERENCE_STATES}')
-    phases = sorted(map(lambda x: x.upper(), phase_models['phases'].keys()))
-    dbf = dbf or Database()
-    dbf.elements.update(set(phase_models['components']))
-    for el in dbf.elements:
-        dbf.species.add(Species(el, {el: 1}, 0))
-        # Add the SER reference data
-        dbf.refstates[el] = espei.refdata.ser_dict[el]
-        # update the refdata for this element with the reference phase
-        if el not in espei.refdata.pure_element_phases.keys():
-            # Probably VA, /- or something else
-            continue
-        refdata_phase = espei.refdata.pure_element_phases[el]
-        if refdata_phase in phases:
-            dbf.refstates[el]['phase'] = refdata_phase
-        else:
-            # Check all the aliases and set the one that matches
-            for phase_name, phase_obj in phase_models['phases'].items():
-                for alias in phase_obj.get('aliases', []):
-                    if alias == refdata_phase:
-                        dbf.refstates[el]['phase'] = phase_name
-    # Write reference state to Database
+    # Set NumPy print options so logged arrays print on one line. Reset at the end.
+    np.set_printoptions(linewidth=sys.maxsize)
+    _log.info('Generating parameters.')
+    _log.trace('Found the following user reference states: %s', espei.refdata.INSERTED_USER_REFERENCE_STATES)
     refdata = getattr(espei.refdata, ref_state)
-    stabledata = getattr(espei.refdata, ref_state + 'Stable')
-    for key, element in refdata.items():
-        if isinstance(element, sympy.Piecewise):
-            newargs = element.args + ((0, True),)
-            refdata[key] = sympy.Piecewise(*newargs)
-    for key, element in stabledata.items():
-        if isinstance(element, sympy.Piecewise):
-            newargs = element.args + ((0, True),)
-            stabledata[key] = sympy.Piecewise(*newargs)
-    comp_refs = {c.upper(): stabledata[c.upper()] for c in dbf.elements if c.upper() != 'VA'}
-    comp_refs['VA'] = 0
-    # note that the `c.upper()*2)[:2]` returns 'AL' for c.upper()=='AL' and 'VV' for c.upper()=='V'
-    dbf.symbols.update({'GHSER' + (c.upper()*2)[:2]: data for c, data in comp_refs.items()})
-    for phase_name, phase_obj in sorted(phase_models['phases'].items(), key=operator.itemgetter(0)):
-        # Perform parameter selection and single-phase fitting based on input
-        # TODO: Need to pass particular models to include: magnetic, order-disorder, etc.
-        symmetry = phase_obj.get('equivalent_sublattices', None)
-        aliases = phase_obj.get('aliases', None)
-        # TODO: More advanced phase data searching
-        site_ratios = phase_obj['sublattice_site_ratios']
-        subl_model = phase_obj['sublattice_model']
-        if phase_name not in dbf.phases.keys():
-            dbf.add_phase(phase_name, dict(), site_ratios)
-            dbf.add_phase_constituents(phase_name, subl_model)
-            dbf.add_structure_entry(phase_name, phase_name)
-        phase_fit(dbf, phase_name, symmetry, subl_model, site_ratios, datasets, refdata, ridge_alpha, aicc_penalty=aicc_penalty_factor, aliases=aliases)
-    logging.info('Finished generating parameters.')
+    aliases = extract_aliases(phase_models)
+    dbf = initialize_database(phase_models, ref_state, dbf)
+    # Fit phases in alphabetic order so the VV#### counter is constistent between runs
+    for phase_name, phase_data in sorted(phase_models['phases'].items(), key=operator.itemgetter(0)):
+        symmetry = phase_data.get('equivalent_sublattices', None)
+        phase_fit(dbf, phase_name, symmetry, datasets, refdata, ridge_alpha, aicc_penalty=aicc_penalty_factor, aliases=aliases)
+    _log.info('Finished generating parameters.')
+    np.set_printoptions(linewidth=75)
     return dbf
