@@ -9,11 +9,14 @@ import matplotlib.lines as mlines
 import numpy as np
 import tinydb
 from pycalphad import Model, calculate, equilibrium, variables as v
+from pycalphad.core.utils import unpack_components
 from pycalphad.plot.utils import phase_legend
 from pycalphad.plot.eqplot import eqplot, _map_coord_to_variable, unpack_condition
 
+from espei.error_functions.non_equilibrium_thermochemical_error import get_prop_samples
 from espei.utils import bib_marker_map
-from espei.core_utils import get_data, get_samples, ravel_zpf_values
+from espei.core_utils import get_data, ravel_zpf_values
+from espei.parameter_selection.utils import _get_sample_condition_dicts
 from espei.sublattice_tools import recursive_tuplify, endmembers_from_interaction
 from espei.utils import build_sitefractions
 
@@ -119,7 +122,7 @@ def plot_parameters(dbf, comps, phase_name, configuration, symmetry, datasets=No
             ax = _compare_data_to_parameters(dbf, comps, phase_name, data, mod, configuration, x_val, y_val, ax=ax)
 
 
-def dataplot(comps, phases, conds, datasets, ax=None, plot_kwargs=None, tieline_plot_kwargs=None):
+def dataplot(comps, phases, conds, datasets, tielines=True, ax=None, plot_kwargs=None, tieline_plot_kwargs=None):
     """
     Plot datapoints corresponding to the components, phases, and conditions.
 
@@ -133,6 +136,8 @@ def dataplot(comps, phases, conds, datasets, ax=None, plot_kwargs=None, tieline_
     conds : dict
         Maps StateVariables to values and/or iterables of values.
     datasets : PickleableTinyDB
+    tielines : bool
+        If True (default), plot the tie-lines from the data
     ax : matplotlib.Axes
         Default axes used if not specified.
     plot_kwargs : dict
@@ -206,8 +211,9 @@ def dataplot(comps, phases, conds, datasets, ax=None, plot_kwargs=None, tieline_
     output = 'ZPF'
     # TODO: used to include VA. Should this be added by default. Can't determine presence of VA in eq.
     # Techincally, VA should not be present in any phase equilibria.
+    # For now, don't get datasets that are a subset of the current system because this breaks mass balance assumptions in ravel_zpf_values
     desired_data = datasets.search((tinydb.where('output') == output) &
-                                   (tinydb.where('components').test(lambda x: set(x).issubset(comps + ['VA']))) &
+                                   (tinydb.where('components').test(lambda x: (set(x).issubset(comps + ['VA'])) and (len(set(x) - {'VA'}) == (len(indep_comps) + 1)))) &
                                    (tinydb.where('phases').test(lambda x: len(set(phases).intersection(x)) > 0)))
 
     # get all the possible references from the data and create the bibliography map
@@ -248,7 +254,8 @@ def dataplot(comps, phases, conds, datasets, ax=None, plot_kwargs=None, tieline_
         updated_tieline_plot_kwargs = {'linewidth':1, 'color':'k'}
         if tieline_plot_kwargs is not None:
             updated_tieline_plot_kwargs.update(tieline_plot_kwargs)
-        equilibria_to_plot = eq_dict.get(2, [])
+        equilibria_to_plot = eq_dict.get(1, [])
+        equilibria_to_plot.extend(eq_dict.get(2, []))
         equilibria_to_plot.extend(eq_dict.get(3, []))
         for eq in equilibria_to_plot:
             # plot the scatter points for the right phases
@@ -267,9 +274,10 @@ def dataplot(comps, phases, conds, datasets, ax=None, plot_kwargs=None, tieline_
                 x_points.append(x_val)
                 y_points.append(y_val)
 
-            # plot the tielines
-            if all([xx is not None and yy is not None for xx, yy in zip(x_points, y_points)]):
-                ax.plot(x_points, y_points, **updated_tieline_plot_kwargs)
+            if tielines and len(x_points) > 1:
+                # plot the tielines
+                if all([xx is not None and yy is not None for xx, yy in zip(x_points, y_points)]):
+                    ax.plot(x_points, y_points, **updated_tieline_plot_kwargs)
 
     elif projection == 'triangular':
         scatter_kwargs = {'markersize': 4, 'markeredgewidth': 0.4}
@@ -286,7 +294,9 @@ def dataplot(comps, phases, conds, datasets, ax=None, plot_kwargs=None, tieline_
         updated_tieline_plot_kwargs = {'linewidth':1, 'color':'k'}
         if tieline_plot_kwargs is not None:
             updated_tieline_plot_kwargs.update(tieline_plot_kwargs)
-        for eq in eq_dict.get(2,[]): # list of things in equilibrium
+        equilibria_to_plot = eq_dict.get(1, [])
+        equilibria_to_plot.extend(eq_dict.get(2, []))
+        for eq in equilibria_to_plot: # list of things in equilibrium
             # plot the scatter points for the right phases
             x_points, y_points = [], []
             for phase_name, comp_dict, ref_key in eq:
@@ -303,9 +313,10 @@ def dataplot(comps, phases, conds, datasets, ax=None, plot_kwargs=None, tieline_
                 x_points.append(x_val)
                 y_points.append(y_val)
 
-            # plot the tielines
-            if all([xx is not None and yy is not None for xx, yy in zip(x_points, y_points)]):
-                ax.plot(x_points, y_points, **updated_tieline_plot_kwargs)
+            if tielines and len(x_points) > 1:
+                # plot the tielines
+                if all([xx is not None and yy is not None for xx, yy in zip(x_points, y_points)]):
+                    ax.plot(x_points, y_points, **updated_tieline_plot_kwargs)
 
 
         # three phase
@@ -382,6 +393,14 @@ def eqdataplot(eq, datasets, ax=None, plot_kwargs=None):
     >>> ax = eqdataplot(eq, datasets, ax=ax)  # doctest: +SKIP
 
     """
+    deprecation_msg = (
+        "`espei.plot.eqdataplot` is deprecated and will be removed in ESPEI 0.9. "
+        "Users depending on plotting from an `pycalphad.equilibrium` result should use "
+        "`pycalphad.plot.eqplot.eqplot` along with `espei.plot.dataplot` directly. "
+        "Note that pycalphad's mapping can offer signficant reductions in calculation "
+        "time compared to using `equilibrium` followed by `eqplot`."
+    )
+    warnings.warn(deprecation_msg, category=FutureWarning)
     # TODO: support reference legend
     conds = OrderedDict([(_map_coord_to_variable(key), unpack_condition(np.asarray(value)))
                          for key, value in sorted(eq.coords.items(), key=str)
@@ -435,6 +454,15 @@ def multiplot(dbf, comps, phases, conds, datasets, eq_kwargs=None, plot_kwargs=N
     >>> multiplot(dbf, ['CU', 'MG', 'VA'], my_phases, {v.P: 101325, v.T: 1000, v.X('MG'): (0, 1, 0.01)}, datasets)  # doctest: +SKIP
 
     """
+    deprecation_msg = (
+        "`espei.plot.multiplot` is deprecated and will be removed in ESPEI 0.9. "
+        "Users depending on `multiplot` should use pycalphad's `binplot` or `ternplot` "
+        "followed by `espei.plot.dataplot`. Note that pycalphad's mapping can offer "
+        "signficant reductions in calculation time compared to using `multiplot`. See "
+        "ESPEI's recipes for an example: "
+        "https://espei.org/en/latest/recipes.html#plot-phase-diagram-with-data"
+    )
+    warnings.warn(deprecation_msg, category=FutureWarning)
     eq_kwargs = eq_kwargs or dict()
     plot_kwargs = plot_kwargs or dict()
     data_kwargs = data_kwargs or dict()
@@ -473,7 +501,14 @@ def _compare_data_to_parameters(dbf, comps, phase_name, desired_data, mod, confi
     matplotlib.Axes
 
     """
-    all_samples = np.array(get_samples(desired_data), dtype=np.object)
+    species = unpack_components(dbf, comps)
+    # phase constituents are Species objects, so we need to be doing intersections with those
+    phase_constituents = dbf.phases[phase_name].constituents
+    # phase constituents must be filtered to only active:
+    constituents = [[sp.name for sp in sorted(subl_constituents.intersection(species))] for subl_constituents in phase_constituents]
+    subl_dof = list(map(len, constituents))
+    calculate_dict = get_prop_samples(desired_data, constituents)
+    sample_condition_dicts = _get_sample_condition_dicts(calculate_dict, subl_dof)
     endpoints = endmembers_from_interaction(configuration)
     interacting_subls = [c for c in recursive_tuplify(configuration) if isinstance(c, tuple)]
     disordered_config = False
@@ -495,7 +530,8 @@ def _compare_data_to_parameters(dbf, comps, phase_name, desired_data, mod, confi
         yattr = y
     if len(endpoints) == 1:
         # This is an endmember so we can just compute T-dependent stuff
-        temperatures = np.array([i[0] for i in all_samples], dtype=np.float)
+        Ts = calculate_dict['T']
+        temperatures = np.asarray(Ts if len(Ts) > 0 else 298.15)
         if temperatures.min() != temperatures.max():
             temperatures = np.linspace(temperatures.min(), temperatures.max(), num=100)
         else:
@@ -556,9 +592,9 @@ def _compare_data_to_parameters(dbf, comps, phase_name, desired_data, mod, confi
 
     for data in desired_data:
         indep_var_data = None
-        response_data = np.zeros_like(data['values'], dtype=np.float)
+        response_data = np.zeros_like(data['values'], dtype=np.float_)
         if x == 'T' or x == 'P':
-            indep_var_data = np.array(data['conditions'][x], dtype=np.float).flatten()
+            indep_var_data = np.array(data['conditions'][x], dtype=np.float_).flatten()
         elif x == 'Z':
             if disordered_config:
                 # Take the second element of the first interacting sublattice as the coordinate
@@ -572,7 +608,7 @@ def _compare_data_to_parameters(dbf, comps, phase_name, desired_data, mod, confi
                     subl_idx = int(subl_idx)
                 indep_var_data = [c[subl_idx][1] for c in occ]
             else:
-                interactions = np.array([i[1][1] for i in get_samples([data])], dtype=np.float)
+                interactions = np.array([cond_dict[Symbol('YS')] for cond_dict in sample_condition_dicts])
                 indep_var_data = 1 - (interactions+1)/2
             if y.endswith('_MIX') and data['output'].endswith('_FORM'):
                 # All the _FORM data we have still has the lattice stability contribution
@@ -590,7 +626,7 @@ def _compare_data_to_parameters(dbf, comps, phase_name, desired_data, mod, confi
                     points[point_idx].update({key: 0 for key in missing_variables})
                     # Change entry to a sorted array of site fractions
                     points[point_idx] = list(OrderedDict(sorted(points[point_idx].items(), key=str)).values())
-                points = np.array(points, dtype=np.float)
+                points = np.array(points, dtype=np.float_)
                 # TODO: Real temperature support
                 points = points[None, None]
                 stability = calculate(dbf, comps, [phase_name], output=data['output'][:-5],
@@ -598,7 +634,7 @@ def _compare_data_to_parameters(dbf, comps, phase_name, desired_data, mod, confi
                                       model=mod_latticeonly, mode='numpy')
                 response_data -= stability[data['output'][:-5]].values.squeeze()
 
-        response_data += np.array(data['values'], dtype=np.float)
+        response_data += np.array(data['values'], dtype=np.float_)
         response_data = response_data.flatten()
         if not bar_chart:
             extra_kwargs = {}
