@@ -6,13 +6,11 @@ Classes and functions defined here should have some reuse potential.
 
 import itertools
 import re
-import os
 from collections import namedtuple
 
 import bibtexparser
 import numpy as np
 import sympy
-import dask
 from bibtexparser.bparser import BibTexParser
 from bibtexparser.customization import convert_to_unicode
 from distributed import Client
@@ -54,6 +52,10 @@ class ImmediateClient(Client):
     returned by map.
     """
     def map(self, f, *iterators, **kwargs):
+        """Map a function on a sequence of arguments.
+
+        Any keyword arguments are passed to distributed.Client.map
+        """
         _client = super(ImmediateClient, self)
         result = _client.gather(_client.map(f, *[list(it) for it in iterators], **kwargs))
         return result
@@ -62,7 +64,7 @@ class ImmediateClient(Client):
 def sigfigs(x, n):
     """Round x to n significant digits"""
     if x != 0:
-        return np.around(x, -(np.floor(np.log10(np.abs(x)))).astype(np.int) + (n - 1))
+        return np.around(x, -(np.floor(np.log10(np.abs(x)))).astype(np.int_) + (n - 1))
     else:
         return x
 
@@ -437,27 +439,30 @@ def popget(d, key, default=None):
         return default
 
 
-def get_dask_config_paths():
-    """
-    Return a list of configuration file paths for dask.
+def extract_aliases(phase_models):
+    """Map possible phase name aliases to the desired phase model phase name.
+    
+    This function enforces that each alias is only claimed by one phase.
+    Each phase name in the phase models is added as an alias for itself to
+    support an "identity" operation.
 
-    The last path in the list has the highest precedence.
+    Parameters
+    ----------
+    phase_models
+        Phase models ESPEI uses to initialize databases. Must contain a mapping
+        of phase names to phase data (sublattices, site ratios, aliases)
 
     Returns
     -------
-    list
+    Dict[str, str]
 
     """
-    candidates = dask.config.paths
-    file_paths = []
-    for path in candidates:
-        if os.path.exists(path):
-            if os.path.isdir(path):
-                file_paths.extend(sorted([
-                    os.path.join(path, p)
-                    for p in os.listdir(path)
-                    if os.path.splitext(p)[1].lower() in ('.json', '.yaml', '.yml')
-                ]))
+    # Intialize aliases with identity for each phase first
+    aliases = {phase_name: phase_name for phase_name in phase_models["phases"].keys()}
+    for phase_name, phase_dict in phase_models["phases"].items():        
+        for alias in phase_dict.get("aliases", []):
+            if alias not in aliases:
+                aliases[alias] = phase_name
             else:
-                file_paths.append(path)
-    return file_paths
+                raise ValueError(f"Cannot add {alias} as an alias for {phase_name} because it is already used by {aliases[alias]}")
+    return aliases
